<?php

class eShopLogistic
{
	private $api_key;
	
	function __construct ($method) {
		$this->api_key = $method->api_key;
		$this->cache_time = $method->cache_time;
		$this->payments = $method->payments;
	}
	
	
	public function get_config() {
		$cache = JFactory::getCache('plg_vmshipment_eshoplogistic', '');
		$cache->setCaching(true);
		$cache->setLifeTime($this->cache_time);// 60 * 60 * 24 = 86400
		$key = 'site';
		if (!($site_config = $cache->get($key))) {
			$data = [
				'key' => $this->api_key,
			];
			
			$curl = curl_init();
			curl_setopt($curl, CURLOPT_URL, 'https://api.eshoplogistic.ru/api/site');
			curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($curl, CURLOPT_TIMEOUT, 10);
			curl_setopt($curl, CURLOPT_POST, 1);
			curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
			$result = curl_exec($curl);
			
			curl_close($curl);
			$site_config = json_decode($result);
			
			$cache->store($site_config, $key);
		}
		
		return $site_config;
	}
	
	
	
	public function search($to, $country = '') {
		$cache = JFactory::getCache('plg_vmshipment_eshoplogistic', '');
		$cache->setCaching(true);
		$cache->setLifeTime(86400);// 60 * 60 * 24 = 86400
		$key = 'search_'.$to.'_'.$country;
		if (!($result = $cache->get($key))) {
			$data = [
				'key' => $this->api_key,
				'target' => $to,
			];
			if (!empty($country)) {
				$data['country'] = $country;
			}
			
			$curl = curl_init();
			curl_setopt($curl, CURLOPT_URL, 'https://api.eshoplogistic.ru/api/search');
			//curl_setopt($curl, CURLOPT_HEADER, 1);
			curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($curl, CURLOPT_TIMEOUT, 10);
			curl_setopt($curl, CURLOPT_POST, 1);
			curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
			$result = curl_exec($curl);
			//var_dump($data, $result);exit;
			curl_close($curl);
			$result = json_decode($result);
			
			$cache->store($result, $key);
		}
		
		return $result;
	}
	
	
	
	public function get_deliveries($cart, $service = null, $to = null) {
		$config = $this->get_config();
		if ($service === null) {
			$service = $_COOKIE['delivery_service_eshoplogistic'];
		}
		if (empty($service) || !count($cart->products)) {
			return null;
		}
		$from = $config->data->services->{$service}->city_code;
		if ($to === null) {
			$to = $_COOKIE['eshoplogistic_city_code'];
		}
		
		$app = JFactory::getApplication();
		
		$offers = array();
		$weight = 0;
		
		$payment = null;
		if (isset($this->payments->{'method_'.$cart->virtuemart_paymentmethod_id})) {
			$payment = $this->payments->{'method_'.$cart->virtuemart_paymentmethod_id};
		}
		
		foreach ($cart->products as $product) {
			$product_lwh_uom = $product->product_lwh_uom; // единицы измерения (размеры)
			if ($product_lwh_uom == "M") $coef_lwh_uom = 0.01;
			if ($product_lwh_uom == "CM") $coef_lwh_uom = 1;
			if ($product_lwh_uom == "MM") $coef_lwh_uom = 10;
			
			$product_length = ($product->product_length) * $coef_lwh_uom;
			$product_width = ($product->product_width) * $coef_lwh_uom;
			$product_height = ($product->product_height)* $coef_lwh_uom;
		
			$dimensions = $product_length.'*'.$product_width.'*'.$product_height;
			
			$item = (object)array(
				"article" => $product->virtuemart_product_id,
				 "name" => $product->product_name,
				 "count" => $product->quantity,
				 "price" => $product->prices['basePriceVariant'],
				 "weight" => $product->product_weight,
				 "dimensions" => $dimensions
			);
			$weight += $product->product_weight * $product->quantity;
			$offers[] = $item;
		}
		$offers = json_encode($offers);
		
		
		$cache = JFactory::getCache('plg_vmshipment_eshoplogistic', '');
		$cache->setCaching(true);
		$cache->setLifeTime(86400);// 60 * 60 * 24 = 86400
		$key = 'deliveries_'.$service.'_'.$from.'_'.$to.'_'.$weight;
		if ($payment) {
			$key .= '_'.$payment;
		}
		$key .= '_'.$offers;
		
		if (!($data_deliveries = $cache->get($key))) {
			$data = [
				'key' => $this->api_key,
				'from' => $from,
				'to' => $to,
				'weight' => $weight,
				'offers' => $offers,
			];
			if ($payment) {
				$data['payment'] = $payment;
			}
			//var_dump($key);//exit;
			
			$curl = curl_init();
			curl_setopt($curl, CURLOPT_URL, 'https://api.eshoplogistic.ru/api/delivery/' . $service);
			//curl_setopt($curl, CURLOPT_HEADER, 1);
			curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($curl, CURLOPT_TIMEOUT, 10);
			curl_setopt($curl, CURLOPT_POST, 1);
			curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
			$result = curl_exec($curl);
			curl_close($curl);
			
			$data_deliveries = json_decode($result);
			//var_dump($data_deliveries);
			$cache->store($data_deliveries, $key);
		}
		
		if (is_object($data_deliveries) && $data_deliveries->success === true) {
			$deliveries = $data_deliveries->data;
			$deliveries->offers = $offers;
		} else {
			$deliveries = false;
		}
		
		return $deliveries;
	}
	
	
	public function rebuild_deliveries($deliveries) {
		$config = $this->get_config();
		$currency = CurrencyDisplay::getInstance();
		
		$new_data = array();
		foreach ($deliveries as $service => $data) {
			if (isset($data->door)) {
				$new_data['door'][$service]['info'] = $data->door;
				$new_data['door'][$service]['info']->logo = $config->data->services->{$service}->logo;
			}
			if (isset($data->terminal->price)) {
				if ($data->terminal->price > 0) {
					$data->terminal->price = $currency->priceDisplay($data->terminal->price);
				} else {
					$data->terminal->price = JText::_('VMSHIPMENT_ESHOPLOGISTIC_PRICE_FREE');
				}
				$data->terminal->name = $config->data->services->{$service}->name;
				$new_data['terminals_info'][$service]['info'] = $data->terminal;
				$new_data['terminals_info'][$service]['info']->logo = $config->data->services->{$service}->logo;
				
				if (isset($data->terminals) && count($data->terminals)) {
					$new_data['terminal'][$service]['terminals'] = $data->terminals;
					if (!isset($new_data['all_terminals'])) {
						$new_data['all_terminals'] = $data->terminals;
					} else {
						$new_data['all_terminals'] = array_merge($new_data['all_terminals'], $data->terminals);
					}
				}
			}
		}
		
		// сортируем по цене
		uasort($new_data['door'], array("eShopLogistic", "sort_services_door"));
		
		return $new_data;
	}
	
	
	// не пашет
	/*
	public function get_deliveries_all($cart) {
		$config = $this->get_config();
		$service = 'all';
		$from = $config->data->settings->city_fias;
		
		$app = JFactory::getApplication();
		$to = $_COOKIE['eshoplogistic_city_fias'];
		
		
		$offers = array();
		$weight = 0;
		
		$payment = null;
		if (isset($this->payments->{'method_'.$cart->virtuemart_paymentmethod_id})) {
			$payment = $this->payments->{'method_'.$cart->virtuemart_paymentmethod_id};
		}
		
		foreach ($cart->products as $product) {
			$product_lwh_uom = $product->product_lwh_uom; // единицы измерения (размеры)
			if ($product_lwh_uom == "M") $coef_lwh_uom = 0.01;
			if ($product_lwh_uom == "CM") $coef_lwh_uom = 1;
			if ($product_lwh_uom == "MM") $coef_lwh_uom = 10;
			
			$product_length = ($product->product_length) * $coef_lwh_uom;
			$product_width = ($product->product_width) * $coef_lwh_uom;
			$product_height = ($product->product_height)* $coef_lwh_uom;
		
			$dimensions = $product_length.'*'.$product_width.'*'.$product_height;
			
			$item = (object)array(
				"article" => $product->virtuemart_product_id,
				 "name" => $product->product_name,
				 "count" => $product->quantity,
				 "price" => $product->prices['basePriceVariant'],
				 "weight" => $product->product_weight,
				 "dimensions" => $dimensions
			);
			$weight += $product->product_weight * $product->quantity;
			$offers[] = $item;
		}
		$offers = json_encode($offers);
		
		
		$cache = JFactory::getCache('plg_vmshipment_eshoplogistic', '');
		$cache->setCaching(true);
		$cache->setLifeTime(86400);// 60 * 60 * 24 = 86400
		$key = 'deliveries_'.$service.'_'.$from.'_'.$to.'_'.$weight;
		if ($payment) {
			$key .= '_'.$payment;
		}
		$key .= '_'.$offers;
		
		if (!($data_deliveries = $cache->get($key))) {
			$data = [
				'key' => $this->api_key,
				'from' => $from,
				'to' => $to,
				'weight' => $weight,
				'offers' => $offers,
			];
			
			$curl = curl_init();
			curl_setopt($curl, CURLOPT_URL, 'https://api.eshoplogistic.ru/api/delivery/' . $service);
			curl_setopt($curl, CURLOPT_SSL_VERIFYHOST, false);
			curl_setopt($curl, CURLOPT_SSL_VERIFYPEER, false);
			curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1);
			curl_setopt($curl, CURLOPT_TIMEOUT, 10);
			curl_setopt($curl, CURLOPT_POST, 1);
			curl_setopt($curl, CURLOPT_POSTFIELDS, $data);
			$result = curl_exec($curl);
			
			curl_close($curl);
			$data_deliveries = (object)array(
				'service' => $service,
				'from' => $from,
				'to' => $to,
				'offers' => $offers,
			);
			$data_deliveries->deliveries = json_decode($result);
			
			$cache->store($data_deliveries, $key);
		}
		
		return $data_deliveries;
	}*/
	
	
	public function get_delivery($deliveries, $terminal_code) {
		//header('Content-type: text/plain');var_dump('1', $deliveries);exit;
		
		if ($terminal_code == 'door' && isset($deliveries->door)) {
			$data = $deliveries->door;
			$data->code = 'door';
			return $data;
		} elseif ($terminal_code == 'postrf') {
			$data = $deliveries->terminal;
			$data->code = 'postrf';
			return $data;
		} elseif (isset($deliveries->terminals) && is_array($deliveries->terminals)) {
			foreach ($deliveries->terminals as $terminal){
				if ($terminal->code == $terminal_code) {
					$data = $deliveries->terminal;
					$data->terminal = $terminal;
					return $data;
				}
			}
		}
	}
	
	
	
	public static function parseStr($str) {
		
		$array_str = explode('|', $str);
		$params = new stdClass;
		foreach ($array_str as $row) {
			if (!empty($row)) {
				list($key, $value) = explode('=', $row);
				$params->$key = trim($value, ' "');
			}
		}
		return $params;
	}
	
	
	
	public static function normalize_utf($str) {
		preg_match_all('/\\\u([0-9a-f]{4})/', $str, $matches);
		foreach ($matches[1] as $hex) {
			$str = str_replace('\u'.$hex, mb_chr(hexdec($hex), 'UTF-8'), $str);
		}
		return $str;
	}
	
	
	
	public static function sort_services_door($a, $b) {
		if ($a['info']->price == $b['info']->price) {
			return 0;
		}
		return ($a['info']->price < $b['info']->price) ? -1 : 1;
	}
}
